function run_test()
{
    load("../../lib/utils.js");

    // Alas, the test framework uses dump(), which croaks on nulls. We must
    // therefore avoid using them in any of the literals that could be
    // displayed. Sigh.
    do_check_eq(ecmaEscape("ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789@*_+-./"), "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789@*_+-./");
    do_check_eq(ecmaEscape("\x01\x02\x03\x04\x05"), "%01%02%03%04%05");
    do_check_eq(ecmaEscape("\u0001\u0002\u0003\u0004\u0005"), "%01%02%03%04%05");
    do_check_eq(ecmaEscape("\u0100\u0101\u0102\u0103\u0104"), "%u0100%u0101%u0102%u0103%u0104");
    do_check_eq(ecmaEscape("\u1000\u1001\u1002\u1003\u1004"), "%u1000%u1001%u1002%u1003%u1004");

    //FIXME:do_check_eq(ecmaUnescape("%1%2%3%4%5"), "%1%2%3%4%5");
    do_check_eq(ecmaUnescape("%01%02%03%04%05"), "\x01\x02\x03\x04\x05");
    //FIXME:do_check_eq(ecmaUnescape("%u100%u101%u102%u103%u104"), "%u100%u101%u102%u103%u104");
    do_check_eq(ecmaUnescape("%u0100%u0101%u0102%u0103%u0104"), "\u0100\u0101\u0102\u0103\u0104");
    do_check_eq(ecmaUnescape("%u1000%u1001%u1002%u1003%u1004"), "\u1000\u1001\u1002\u1003\u1004");
    do_check_eq(ecmaUnescape("%u10000%u10001%u10002%u10003%u10004"), "\u1000"+"0\u1000"+"1\u1000"+"2\u1000"+"3\u1000"+"4");
    do_check_eq(ecmaUnescape("%u100000%u100001%u100002%u100003%u100004"), "\u1000"+"00\u1000"+"01\u1000"+"02\u1000"+"03\u1000"+"04");

    do_check_eq(encodeForXMLAttribute("&<>\"'&<>\"'"), "&amp;&lt;&gt;&quot;&apos;&amp;&lt;&gt;&quot;&apos;");
    do_check_eq(encodeForXMLAttribute("&&&amp;"), "&amp;&amp;&amp;amp;");

    var o1 = { foo: 42 };
    do_check_eq(replaceVars("foo", o1), "foo");
    do_check_eq(replaceVars("foo$foofoo", o1), "foo$foofoo");
    do_check_eq(replaceVars("foo $foo foo", o1), "foo 42 foo");
    do_check_eq(replaceVars("$foo foo $foo", o1), "42 foo 42");
    o1["foo-bar"] = 43;
    do_check_eq(replaceVars("$foo $foo-bar $bar", o1), "42 43 $bar");
    o1["-bar"] = 44;
    do_check_eq(replaceVars("foo $-bar foo", o1), "foo $-bar foo");
    o1["42"] = 45;
    o1["f42"] = 46;
    //FIXME:do_check_eq(replaceVars("$42 $f42", o1), "$42 46");

    o1 = Clone({ foo: 42, bar: 43 });
    do_check_eq(o1.foo, 42);
    do_check_eq(o1.bar, 43);
    var o2 = { baz: 44 };
    o2.__proto__ = o1;
    var o3 = Clone(o2);
    do_check_eq(o3.foo, 42);
    do_check_eq(o3.bar, 43);
    do_check_eq(o3.baz, 44);
    do_check_false(o3.hasOwnProperty("foo"));
    do_check_false(o3.hasOwnProperty("bar"));
    do_check_true(o3.hasOwnProperty("baz"));
    do_check_eq(o3.__proto__, o1);

    o1 = { foo: 42 };
    o2 = Copy(o1);
    do_check_eq(o2.foo, 42); // Property is copied.
    o2 = Copy(o1, { foo: 43, bar: 44 });
    do_check_eq(o2.foo, 43); // Property is NOT overwritten.
    do_check_eq(o2.bar, 44); // Property is untouched.
    o2 = Copy(o1, { foo: 43, bar: 44 }, true);
    do_check_eq(o2.foo, 42); // Property IS overwritten.
    do_check_eq(o2.bar, 44); // Property is untouched.

    o1 = { foo: 42, bar: "baz", baz: false };
    // As single object.
    do_check_true(matchObject(o1, {}));
    do_check_true(matchObject(o1, { foo: "" }));
    do_check_true(matchObject(o1, { foo: 2 }));
    do_check_true(matchObject(o1, { foo: 4 }));
    do_check_true(matchObject(o1, { foo: 42 }));
    do_check_false(matchObject(o1, { foo: 43 }));
    do_check_true(matchObject(o1, { bar: "baz" }));
    do_check_true(matchObject(o1, { foo: 42, bar: "baz", baz: false }));
    do_check_false(matchObject(o1, { foo: 42, bar: "foo|bar", baz: false }));
    do_check_true(matchObject(o1, { foo: 42, bar: "foo|bar|baz", baz: false }));
    do_check_false(matchObject(o1, { foo: 42, bar: "fail", baz: false }));
    do_check_false(matchObject(o1, { foo: 44, bar: "fail", baz: true }));
    // As array with one element.
    do_check_true(matchObject(o1, [{}]));
    do_check_true(matchObject(o1, [{ foo: 2 }]));
    do_check_true(matchObject(o1, [{ foo: 4 }]));
    do_check_true(matchObject(o1, [{ foo: 42 }]));
    do_check_false(matchObject(o1, [{ foo: 43 }]));
    do_check_true(matchObject(o1, [{ bar: "baz" }]));
    do_check_true(matchObject(o1, [{ foo: 42, bar: "baz", baz: false }]));
    do_check_true(matchObject(o1, [{ foo: 42, bar: "baz", baz: false }]));
    do_check_false(matchObject(o1, [{ foo: 42, bar: "fail", baz: false }]));
    do_check_false(matchObject(o1, [{ foo: 44, bar: "fail", baz: true }]));
    // As array with multiple elements.
    do_check_true(matchObject(o1, [{}, {}]));
    do_check_true(matchObject(o1, [{ foo: 42 }, { foo: 43 }]));
    do_check_true(matchObject(o1, [{ foo: 43 }, { foo: 42 }]));
    do_check_false(matchObject(o1, [{ foo: 43 }, { foo: 43 }]));
    // Now with functions.
    do_check_true(matchObject(o1, function(object) { return object.foo == 42 }));
    do_check_false(matchObject(o1, function(object) { return object.foo == 43 }));
    do_check_true(matchObject(o1, [function(object) { return object.foo == 42 }]));
    do_check_false(matchObject(o1, [function(object) { return object.foo == 43 }]));
    do_check_true(matchObject(o1, [function(object) { return object.foo == 42 }, function(object) { return object.foo == 43 }]));
    do_check_true(matchObject(o1, [function(object) { return object.foo == 43 }, function(object) { return object.foo == 42 }]));
    do_check_false(matchObject(o1, [function(object) { return object.foo == 43 }, function(object) { return object.foo == 43 }]));

    do_check_true(equalsObject({}, {}));
    do_check_false(equalsObject({}, { foo: 42 }));
    do_check_false(equalsObject({ foo: 42 }, {}));
    do_check_true(equalsObject({ foo: 42 }, { foo: 42 }));
    do_check_false(equalsObject({ foo: 42 }, { foo: 43 }));
    do_check_false(equalsObject({ foo: 42 }, { foo: 4 }));
    do_check_true(equalsObject({ foo: "bar" }, { foo: "bar" }));
    do_check_false(equalsObject({ foo: "bar" }, { foo: "bar " }));
    do_check_false(equalsObject({ foo: "bar" }, { foo: "baz" }));

    // Special note: this group of tests rely on equalsObject(), tested above.
    do_check_true(equalsObject(matchEntry(void 0, []), []));
    do_check_true(equalsObject(matchEntry(void 0, ["1"]), [0]));
    do_check_true(equalsObject(matchEntry(void 0, ["1", "2"]), [0, 1]));
    do_check_true(equalsObject(matchEntry("", []), []));
    do_check_true(equalsObject(matchEntry("", ["1"]), [0]));
    do_check_true(equalsObject(matchEntry("", ["1", "2"]), [0, 1]));
    o1 = ["foo", "bar", "baz"];
    do_check_true(equalsObject(matchEntry("", o1), [0, 1, 2]));
    do_check_true(equalsObject(matchEntry("f", o1), [0]));
    do_check_true(equalsObject(matchEntry("F", o1), [0]));
    do_check_true(equalsObject(matchEntry("foo", o1), [0]));
    do_check_true(equalsObject(matchEntry("FOO", o1), [0]));
    do_check_true(equalsObject(matchEntry("b", o1), [1, 2]));
    do_check_true(equalsObject(matchEntry("B", o1), [1, 2]));
    do_check_true(equalsObject(matchEntry("bar", o1), [1]));
    do_check_true(equalsObject(matchEntry("BAR", o1), [1]));
    do_check_true(equalsObject(matchEntry("o", o1), []));
    do_check_true(equalsObject(matchEntry("O", o1), []));
    do_check_true(equalsObject(matchEntry("oo", o1), []));
    do_check_true(equalsObject(matchEntry("OO", o1), []));

    do_check_eq(escapeFileName("filename.extension"), "filename.extension");
    do_check_eq(escapeFileName("/\\:*?\"<>|"), "%2f%5c%3a%2a%3f%22%3c%3e%7c");

    do_check_eq(getCommonPfx([]), void 0);
    do_check_eq(getCommonPfx(["foo", "bar"]), "");
    do_check_eq(getCommonPfx(["bar", "baz"]), "ba");
    do_check_eq(getCommonPfx(["BaR", "bAz"]), "Ba");
    do_check_eq(getCommonPfx(["bAz", "BaR"]), "bA");
    do_check_eq(getCommonPfx(["BAR", "BAZ", "baar"]), "BA");

    // FIXME: Test renameProperty()

    var o1 = ["A", "B", "C"];
    do_check_false(arrayHasElementAt(o1, -1));
    do_check_true(arrayHasElementAt(o1, 0));
    do_check_true(arrayHasElementAt(o1, 1));
    do_check_true(arrayHasElementAt(o1, 2));
    do_check_false(arrayHasElementAt(o1, 3));

    do_check_true(arrayContains(o1, "A"));
    do_check_true(arrayContains(o1, "B"));
    do_check_true(arrayContains(o1, "C"));
    do_check_false(arrayContains(o1, "a"));
    do_check_false(arrayContains(o1, "f"));

    do_check_eq(arrayIndexOf(o1, "A"), 0);
    do_check_eq(arrayIndexOf(o1, "B"), 1);
    do_check_eq(arrayIndexOf(o1, "C"), 2);
    do_check_eq(arrayIndexOf(o1, "a"), -1);
    do_check_eq(arrayIndexOf(o1, "f"), -1);

    o2 = ["A", "B", "C"]; arrayInsertAt(o2, 0, "I");
    do_check_true(equalsObject(o2, ["I", "A", "B", "C"]));
    o2 = ["A", "B", "C"]; arrayInsertAt(o2, 2, "I");
    do_check_true(equalsObject(o2, ["A", "B", "I", "C"]));
    o2 = ["A", "B", "C"]; arrayInsertAt(o2, 3, "I");
    do_check_true(equalsObject(o2, ["A", "B", "C", "I"]));

    o2 = ["A", "B", "C"]; arrayRemoveAt(o2, 0);
    do_check_true(equalsObject(o2, ["B", "C"]));
    o2 = ["A", "B", "C"]; arrayRemoveAt(o2, 2);
    do_check_true(equalsObject(o2, ["A", "B"]));
    o2 = ["A", "B", "C"]; arrayRemoveAt(o2, 3);
    do_check_true(equalsObject(o2, ["A", "B", "C"]));

    // FIXME: Test abbreviateWord()
    // FIXME: Test hyphenateWord()
    // FIXME: Test getRandomElement()
    // FIXME: Test roundTo()
    // FIXME: Test randomRange()
    // FIXME: Test randomString()
    // FIXME: Test getStackTrace()
    // FIXME: Test mapObjFunc()
    // FIXME: Test map()
    // FIXME: Test getSpecialDirectory()
    // FIXME: Test getFileFromURLSpec()
    // FIXME: Test getURLSpecFromFile()
    // FIXME: Test getHostmaskParts()
    // FIXME: Test hostmaskMatches()
    // FIXME: Test isinstance()
    // FIXME: Test scaleNumberBy1024()
    // FIXME: Test getSISize()
    // FIXME: Test getSISpeed()
    // FIXME: Test compareVersions()
    // FIXME: Test padNumber()
    // FIXME: Test strftime()
}
